@@ -505,3 +505,38 @@ on_0x3_activate                        (GtkMenuItem     *menuitem,
 void
 on_0x4_activate                        (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_Gen_p_clicked                       (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_clist2_select_row                   (GtkCList        *clist,
+                                        gint             row,
+                                        gint             column,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
+
+void
+on_button84_clicked                    (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_button87_clicked                    (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_radiobutton61_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_radiobutton62_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_radiobutton61_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_radiobutton62_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
