@@ -1,6 +1,6 @@
 /*
  * packETH - ethernet packet generator
- * By Miha Jemec <m.jemec@iskratel.si>
+ * By Miha Jemec <jemcek@gmail.com>
  * Copyright 2003 Miha Jemec, Iskratel
  *
  * This program is free software; you can redistribute it and/or
@@ -29,8 +29,10 @@
 #include "support.h"
 #include "function.h"
 #include "callbacks.h"
+//#include "libnet/libnet-headers.h"
+//#include "headers.h"
 
-static GtkWidget *w1, *w2, *w3, *w4, *w5, *w6, *w7;
+static GtkWidget *w1, *w2, *w3, *w4, *w5, *w6, *w7, *w8;
 char field[3100];
 char temp[20];
 char *ptrf;
@@ -38,7 +40,7 @@ char *ptrt;
 int next_prot;
 long i;
 int remain;
-
+protocol_type protokol;
 
 /* this one loads the parameters from file into notebook2 (Gen-b page) */
 int load_gen_b_data(GtkButton *button, FILE *file_p) {
@@ -152,7 +154,7 @@ int load_gen_b_data(GtkButton *button, FILE *file_p) {
 int load_gen_s_data(GtkButton *button, FILE *file_p) {
 
 	long int buff4[5];
-        char buff[10];
+        char buff[100];
         char buffc[11][200];
 	int c, j, k;
 	char *ptr = NULL, *ptr2 = NULL;
@@ -335,53 +337,220 @@ int load_gen_s_data(GtkButton *button, FILE *file_p) {
 
 }
 
-
-/* this one loads the parameters from file into notenook2 (Builder page) */
-int load_packet_disector(GtkButton *button, FILE *fp) {
-
-	int j=0, c;
-	/* we load the packet contents from file, skiping comments and new lines */
-	while ( (c = fgetc( fp )) != EOF ) {
-		/* skip all the comment lines, starting with # */
-		if ( c == 35) {
-			while ( getc(fp) != 10);
-				continue;
-		}
-		/* and new lines */
-		if (c == 10) 
-			continue;
-		/* now the info: only hexadecimal digits are acceptable */
-		if (isxdigit(c) != 0) {
-			field[j] = c;
-			j++;
-			field[j] = '\0';
-		}
-		else {
-			error("Can't load packet: Data has wrong format!");
-			return -1;
-		}
-		/* we check the length here: 1514 bytes or 1518 in case with VLAN field is max allowed
-		 * since we don't know yet if there is vlan field included we only raise 
-		 * an error if it is longer than 1518 bytes (1518*2 + \0 = 3037) 
-		 * before sending the length is examined again anyway */
-		if (j>3037) {
-			error("Can't load packet: Data is to long!");
-			return -1;
-		}
-	}
+/* opens the pcap file. 
+ * if we call this function from the builder window only one - first, packet will be read 
+ * from the Genp window, packets will be loaded untill EOF is reached */
+int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets) {
 	
-	remain = j; /* because j starts with 0, we don't need to subtract -1 for \0 */
+	struct pcap_hdr fh;
+	struct pcaprec_hdr ph;
+	struct clist_hdr clh;
+	int j, ji, freads;
+	guint32 secu = 0, secb = 0;
+	guint32 usecu = 0, usecb = 0;
+	double timediff = 0;
+	double timebeg= 0;
+	char pkt_temp[3100];
+	GtkWidget *clis;
 
-	if (remain%2 != 0) {
-		error("Can't load packet: Data length is not an even number!");
+	/* first we read the pcap file header */
+	freads = fread(pkt_temp, sizeof(fh), 1, file_p);
+	/* if EOF, exit */
+	if (freads == 0)
+		return 1;
+
+	memcpy(&fh, pkt_temp, 24);
+
+	/* if magic number in NOK, exit */
+	if (fh.magic != PCAP_MAGIC) 
 		return -1;
-	}
-	remain = remain/2;
 
-	/* this is how it looks like */
-	//for (j=0; field[j]!='\0'; j++)
-	//	printf("%x", field[j]); 
-	//printf("\n");
+	/* hm, I  forgot a little bit, but I assume 1 means builder here */
+	if (whocalled == 1) {
+		
+		/* next the  pcap packet header */
+		freads = fread(pkt_temp, sizeof(ph), 1, file_p);
+		
+		/* if EOF, exit */
+		if (freads == 0)
+			return 1;
+
+		/* copy the 16 bytes into ph structure */
+		memcpy(&ph, pkt_temp, 16);
+
+		/* and the packet itself, but only up to the capture length */
+		freads = fread(pkt_temp+sizeof(ph), ph.incl_len, 1, file_p);
+
+		/* if EOF, exit */
+		if (freads == 0)
+			return 1;
+
+		/* convert the packet information from int to hex */
+		for(ji=0; ji<(sizeof(ph)+ph.incl_len); ji++)
+			c8(&field[2*ji], *(pkt_temp+ji)); 
+		field[2*ji+2] = '\0';
+		
+		load_packet_disector(button, field, 1, &clh, ph.incl_len);			
+
+		return 1; 
+	}
+
+	else { 
+	
+		clis = lookup_widget(GTK_WIDGET (button), "clist2");
+		gtk_clist_clear(GTK_CLIST(clis));
+	
+		for (j=0; j<howmanypackets; j++) {
+	
+			/* next the  pcap packet header */
+			fread(pkt_temp, sizeof(ph), 1, file_p);
+
+			/* if EOF, exit */
+			if (freads == 0)
+				return 1;
+
+			/* copy the 16 bytes into ph structure */
+			memcpy(&ph, pkt_temp, 16);
+
+			/* and the packet itself, but only up to the capture length */
+			freads = fread(pkt_temp+sizeof(ph), ph.incl_len, 1, file_p);
+
+			/* if EOF, exit */
+			if (freads == 0)
+				return 1;
+
+			/* convert the packet information from int to hex */
+			for(ji=0; ji<(sizeof(ph)+ph.incl_len); ji++)
+				c8(&field[2*ji], *(pkt_temp+ji)); 
+			field[2*ji+2] = '\0';
+		
+			/* we have to dissect the packet to get information for the list */
+			load_packet_disector(button, field, 2, &clh, ph.incl_len);			
+			//printf("tukaj3 %s\n", field);
+			
+			/* calculate the time information */
+			if (j==0) { 
+				timediff = 0;
+				timebeg = 0;
+				secb = ph.ts_sec;
+				usecb = ph.ts_usec;
+			}
+			else { 
+				timediff = ((double)((double)ph.ts_sec - (double)secu)*1000000 + 
+								(double)((double)ph.ts_usec - (double)usecu)) / 1000000;
+				timebeg = ((double)((double)ph.ts_sec - (double)secb)*1000000 + 
+								(double)((double)ph.ts_usec - (double)usecb)) / 1000000;
+			}
+
+			//printf("tukaj2 %s\n", field);
+
+			secu = ph.ts_sec;
+			usecu = ph.ts_usec;
+
+			/* insert a new row into clist */
+			load_gen_p_data(button, clis, field, &ph, j+1, &clh, timediff, timebeg);
+		}
+		if (j == howmanypackets) 
+			error("Only first 1000 packets loaded!\nTo change this modify #define on top of callbacks.c");
+	}
+
+	return 1;
+
+}
+	
+
+/* this one loads the parameters from file into notebook2 (Genp page) */
+int load_gen_p_data(GtkButton *button, GtkWidget *clis, char *fieldek, struct pcaprec_hdr *ph2, 
+					int pkt_nr, struct clist_hdr *clptr, double timediff, double timebeg) {
+
+	gchar *datap[8];
+	gchar fieldp[7][21];
+	gchar field_p[1][3100];
+
+	datap[0]=&fieldp[0][0];
+	datap[1]=&fieldp[1][0];
+	datap[2]=&fieldp[2][0];
+	datap[3]=&fieldp[3][0];
+	datap[4]=&fieldp[4][0];
+	datap[5]=&fieldp[5][0];
+	datap[6]=&fieldp[6][0];
+	datap[7]=&field_p[0][0];
+
+	//printf("TUKAJ1:%s\n", fieldek);
+	g_snprintf(fieldp[0], 20, "%d", pkt_nr);
+	g_snprintf(fieldp[1], 20, "%f", timebeg);
+	g_snprintf(fieldp[2], 20, "%f", timediff);
+
+	if ( (*ph2).incl_len == (*ph2).orig_len )
+		g_snprintf(fieldp[3], 20, "%d", (*ph2).incl_len);
+	else
+		g_snprintf(fieldp[3], 20, "%d !", (*ph2).incl_len);
+
+	g_snprintf(fieldp[4], 20, "%s", clptr->src);
+	g_snprintf(fieldp[5], 20, "%s", clptr->dst);
+
+	switch (protokol) {
+		case ETH_II: {
+			g_snprintf(fieldp[6], 20, "Ethernet II");
+			break;
+		}
+		case ETH_802_3: {
+			g_snprintf(fieldp[6], 20, "Ethernet 802.3");
+			break;
+		}
+		case ARP: {
+			g_snprintf(fieldp[6], 20, "ARP");
+			break;
+		}
+		case IPv4: {
+			g_snprintf(fieldp[6], 20, "IPv4");
+			break;
+		}
+		case TCP: {
+			g_snprintf(fieldp[6], 20, "TCP");
+			break;
+		}
+		case UDP: {
+			g_snprintf(fieldp[6], 20, "UDP");
+			break;
+		}
+		case IGMP: {
+			g_snprintf(fieldp[6], 20, "IGMP");
+			break;
+		}
+		case ICMP: {
+			g_snprintf(fieldp[6], 20, "ICMP");
+			break;
+		}
+	}
+
+	g_snprintf(field_p[0], 2*(32+(*ph2).incl_len), "%s", fieldek);
+		
+	gtk_clist_append(GTK_CLIST(clis), datap);
+	
+	return 1;
+
+}
+
+/* this routine was changed. it loads the packet into notebook2 (Builder page) or it checks the file containing the packets
+ * for loading into Genp window
+ * if who called = 1 - we load the contents into builder field
+ * if who called = 2 - we load the contenst into Genp window but we need the information for filling in the clist 
+ */
+int load_packet_disector(GtkButton *button, char *fieldek, int whocalled, struct clist_hdr *clptr, int dolpaketa) {
+
+	int c;
+
+	ptrf = fieldek;
+	ptrt = temp;
+
+	//printf("\n:\n%s\n", fieldek);
+	//convert8field(ptrt, ptrf);    insint(button, "entry179", ptrt, 8); ptrt = temp; ptrf = ptrf-8;
+	//convert8field(ptrt, ptrf+8);  insint(button, "entry180", ptrt, 8); ptrt = temp; ptrf = ptrf-8;
+	//convert8field(ptrt, ptrf+16); insint(button, "entry181", ptrt, 8); ptrt = temp; ptrf = ptrf-8;
+	//convert8field(ptrt, ptrf+24); insint(button, "entry182", ptrt, 8); ptrt = temp; ptrf = ptrf-8;
+	remain = dolpaketa;
+	ptrf = ptrf + 32;
 
 	/* what is the shortest length we still allow?
 	 * we don't care if the packet is shorter than actually allowed to go on ethernet
@@ -393,9 +562,6 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		return -1;
 	}
 
-	ptrf = field;
-	ptrt = temp;
-
 	/* first there is destination mac */
 	w1 = lookup_widget(GTK_WIDGET(button), "L_dst_mac");
 	for (i=1; i<=18; i++, ptrt++) {
@@ -405,8 +571,12 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 			*ptrt = *ptrf++;
 		}
 	}
-	*ptrt = '\0';
-	gtk_entry_set_text(GTK_ENTRY(w1), temp);
+	*(ptrt-1) = '\0';
+
+	if (whocalled == 1)
+		gtk_entry_set_text(GTK_ENTRY(w1), temp);
+	else 
+		memcpy(clptr->dst, temp, 20);
 
 	/* and source mac */
 	ptrt = temp;
@@ -418,10 +588,14 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 			*ptrt = *ptrf++;
 		}
 	}
-	*ptrt = '\0';
-	gtk_entry_set_text(GTK_ENTRY(w2), temp);
+	*(ptrt-1) = '\0';
 
-	/* next there is type or length field or 802.1q */
+	if (whocalled == 1)
+		gtk_entry_set_text(GTK_ENTRY(w2), temp);
+	else 
+		memcpy(clptr->src, temp, 20);
+
+	/* next there is type or length field or 802.1q or QinQ! */
 	i = char2x(ptrf)*256 + char2x(ptrf+2);
 
 	remain = remain - 14;
@@ -429,7 +603,47 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 	/* in case of a vlan tag 0x8100 == 33024) */
 	w1 = lookup_widget(GTK_WIDGET(button), "bt_8021q");
 	w2 = lookup_widget(GTK_WIDGET(button), "frame6");
-	if (i == 33024) {
+
+	if ((i == 33024) || (i==34984) || (i==37120) || (i==37376)) {
+		w3 = lookup_widget(GTK_WIDGET(button), "L_optmenu2_bt");
+		w4 = lookup_widget(GTK_WIDGET(button), "checkbutton39");
+		w5 = lookup_widget(GTK_WIDGET(button), "checkbutton40");
+		w6 = lookup_widget(GTK_WIDGET(button), "L_vlan_id");
+		w7 = lookup_widget(GTK_WIDGET(button), "entry165");
+		w8 = lookup_widget(GTK_WIDGET(button), "optionmenu21");
+		
+	        //if we have 8100 after the next 4 bytes we do QinQ	
+		if ( ((char2x(ptrf+8)*256 + char2x(ptrf+10))==33024) && remain>=8) 
+		{
+			ptrf = ptrf + 4;
+			
+			if (whocalled == 1) {
+				gtk_widget_set_sensitive (w7, TRUE);
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), TRUE);
+				if (i==33024)
+					gtk_option_menu_set_history (GTK_OPTION_MENU (w8), 0);
+				else if (i==34984)
+					gtk_option_menu_set_history (GTK_OPTION_MENU (w8), 3);
+				else if (i==37120)
+					gtk_option_menu_set_history (GTK_OPTION_MENU (w8), 1);
+				else if (i==37376)
+					gtk_option_menu_set_history (GTK_OPTION_MENU (w8), 2);
+
+				inspar(button, "entry165", ptrf, 4);
+			}
+			else
+				ptrf = ptrf +3;
+
+			i = char2x(ptrf)*256 + char2x(ptrf+2);
+		}	
+		else {
+			if (whocalled == 1) {
+				gtk_widget_set_sensitive (w7, FALSE);
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), FALSE);
+			}	
+
+		}
+
 		if (remain < 4) {
 			error("Can't load packet: Ethernet VLAN field is not long enough!");
 			return -1;
@@ -437,55 +651,34 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		remain = remain -4;
 
 		ptrf = ptrf + 4;
-		w3 = lookup_widget(GTK_WIDGET(button), "L_optmenu2_bt");
-		w4 = lookup_widget(GTK_WIDGET(button), "checkbutton39");
-		w5 = lookup_widget(GTK_WIDGET(button), "checkbutton40");
-		w6 = lookup_widget(GTK_WIDGET(button), "L_vlan_id");
-		w7 = lookup_widget(GTK_WIDGET(button), "entry165");
-		
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
-		gtk_widget_set_sensitive (w2, TRUE);
-
 		*ptrt++ = '0';
 		*ptrt-- = *ptrf++;
 		i = char2x(ptrt);
-		gtk_option_menu_set_history (GTK_OPTION_MENU (w3), (i>>1));
 
-		if ( (i%2) == 0)
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), FALSE);
+		if (whocalled == 1) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+			gtk_widget_set_sensitive (w2, TRUE);
+		
+			gtk_option_menu_set_history (GTK_OPTION_MENU (w3), (i>>1));
+
+			if ( (i%2) == 0)
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), FALSE);
+			else
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), TRUE);
+
+			inspar(button, "L_vlan_id", ptrf, 3);
+		}
 		else
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), TRUE);
-
-		inspar(button, "L_vlan_id", ptrf, 3);
+			ptrf = ptrf+3;
 
 		i = char2x(ptrf)*256 + char2x(ptrf+2);
 
-		/* we support now vlan stacking, in case the protocol in once again 0x8100
-		 * we just add next 4 bytes in a QinQ field and switch the toggle button */
-		if (i == 33024) {
-			if (remain < 4) {
-				error("Can't load packet: QinQ VLAN field is not long enough!");
-				return -1;
-			}
-			remain = remain -4;
-
-			ptrf = ptrf + 4;
-			
-			gtk_widget_set_sensitive (w7, TRUE);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), TRUE);
-
-			inspar(button, "entry165", ptrf, 4);
-
-			i = char2x(ptrf)*256 + char2x(ptrf+2);
-		}	
-		else {
-			gtk_widget_set_sensitive (w7, FALSE);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), FALSE);
-		}
 	}
 	else {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
-		gtk_widget_set_sensitive (w2, FALSE);
+		if (whocalled == 1) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
+			gtk_widget_set_sensitive (w2, FALSE);
+		}	
 	}
 	
 	/* c will tell us which ethernet type we have */
@@ -494,7 +687,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 	/* ok, from now one, we split the dissection in different routines, depending on what values */
 	/* now if length is <= 1500, we have 802.3 ethernet and this value means length of ethernet packet */
 	if (i <= 1500) 
-		next_prot = ethernet_8023(button);
+		next_prot = ethernet_8023(button, whocalled);
 	/* Values between 1500 and 1536 are forbidden */
 	else if ( (i>1500) && (i<1536) ) {
 		error("Can't load packet: Wrong ethernet length/type field");
@@ -502,7 +695,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 	}
 	/* if i >= 1536 - ethernet ver II */
 	else
-		next_prot = ethernet_verII(button);
+		next_prot = ethernet_verII(button, whocalled);
 
 
 	/* ok, so we have dissected the ethernet layer and now move on two the next layer.
@@ -520,7 +713,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 	/* ipv4 */
 	else if (next_prot == 2048) {
 		/* ok, ipv4 should follow, so we call the routine for parsing ipv4 header. */
-		next_prot = ipv4_header(button);
+		next_prot = ipv4_header(button, whocalled, clptr);
 		if (next_prot == -1)
 			return -1;
 
@@ -533,7 +726,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		/* here we do the further parsing: tcp, udp, icmp, ...*/
 		if (next_prot == 1) {
 			/* try to parse icmp header */
-			next_prot = icmp_header(button);
+			next_prot = icmp_header(button, whocalled);
 			/* not ok, return an error */
 			if (next_prot == -1)
 				return -1;
@@ -545,7 +738,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		}
 		else if (next_prot == 2) {
 			/* try to parse igmp header */
-			next_prot = igmp_header(button);
+			next_prot = igmp_header(button, whocalled);
 			/* not ok, return an error */
 			if (next_prot == -1)
 				return -1;
@@ -557,7 +750,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		}
 		else if (next_prot == 6) {
 			/* try to parse tcp header */
-			next_prot = tcp_header(button);
+			next_prot = tcp_header(button, whocalled);
 			/* not ok, return an error */
 			if (next_prot == -1)
 				return -1;
@@ -572,7 +765,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		}	
 		else if (next_prot == 17) {
 			/* try to parse udp header */
-			next_prot = udp_header(button);
+			next_prot = udp_header(button, whocalled);
 			/* not ok, return an error */
 			if (next_prot == -1)
 				return -1;
@@ -586,7 +779,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 		}	
 		/* protocol we do not support yet; user defined window */
 		else {
-			next_prot = usedef_insert(button, "text2");
+			next_prot = usedef_insert(button, "text2", whocalled);
 			w1 = lookup_widget(GTK_WIDGET(button), "ip_user_data_bt");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
 		}
@@ -594,7 +787,7 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 	/*arp */
 	else if (next_prot == 2054) {
 		/* ok, arp header follows */
-		next_prot = arp_header(button);
+		next_prot = arp_header(button, whocalled);
 		if (next_prot == -1)
 			return -1;
 
@@ -637,18 +830,23 @@ int load_packet_disector(GtkButton *button, FILE *fp) {
 			inspar(button, "L_pid", ptrf, 4);
 		}		
 
-		next_prot = usedef_insert(button, "text1");
+		next_prot = usedef_insert(button, "text1", whocalled);
 	}
 
 	return 1;
 }
 
 
-int arp_header(GtkButton *button) {
+int arp_header(GtkButton *button, int whocalled) {
 
-	char tmp[4];
+	char tmp[5];
 	int x;
 
+	if (whocalled==2) {
+		protokol = ARP;
+		return 1;
+	}
+
 	/* arp header length == 28; but packet can be longer, f.e. to satisfy the min packet length */
 	if (remain < 28) {
 		error("Can't load packet: Packet length shorter than ARP header length!");
@@ -707,9 +905,9 @@ int arp_header(GtkButton *button) {
 		if (i%3 == 0) {			
 			x = char2x(tmp);
 			if (i==12)
-				sprintf(tmp, "%d", x);		
+				snprintf(tmp, 4, "%d", x);		
 			else
-				sprintf(tmp, "%d.", x);		
+				snprintf(tmp, 5, "%d.", x);		
 			strcat(temp, tmp);
 		}
 		else {
@@ -739,9 +937,9 @@ int arp_header(GtkButton *button) {
 		if (i%3 == 0) {			
 			x = char2x(tmp);
 			if (i==12)
-				sprintf(tmp, "%d", x);		
+				snprintf(tmp, 4, "%d", x);		
 			else
-				sprintf(tmp, "%d.", x);		
+				snprintf(tmp, 5, "%d.", x);		
 			strcat(temp, tmp);
 		}
 		else {
@@ -755,10 +953,15 @@ int arp_header(GtkButton *button) {
 }
 
 
-int igmp_header(GtkButton *button) {
+int igmp_header(GtkButton *button, int whocalled) {
 
 	int x, x1;
-	char tmp[4];
+	char tmp[5];
+
+	if (whocalled==2) {
+		protokol = IGMP;
+		return 1;
+	}
 
 	/* well normal igmp type should have at least 8 bytes, so this is min for us */
 	if (remain < 8) {
@@ -823,9 +1026,9 @@ int igmp_header(GtkButton *button) {
 			if (i%3 == 0) {			
 				x = char2x(tmp);
 				if (i==12)
-					sprintf(tmp, "%d", x);		
+					snprintf(tmp, 4, "%d", x);		
 				else
-					sprintf(tmp, "%d.", x);		
+					snprintf(tmp, 5, "%d.", x);		
 				strcat(temp, tmp);
 			}
 			else {
@@ -857,9 +1060,9 @@ int igmp_header(GtkButton *button) {
 			if (i%3 == 0) {			
 				x = char2x(tmp);
 				if (i==12)
-					sprintf(tmp, "%d", x);		
+					snprintf(tmp, 4, "%d", x);		
 				else
-					sprintf(tmp, "%d.", x);		
+					snprintf(tmp, 5, "%d.", x);		
 				strcat(temp, tmp);
 			}
 			else {
@@ -874,10 +1077,15 @@ int igmp_header(GtkButton *button) {
 	return 1;
 }
 
-int icmp_header(GtkButton *button) {
+int icmp_header(GtkButton *button, int whocalled) {
 
 	int x;
 
+	if (whocalled==2) {
+		protokol = ICMP;
+		return 1;
+	}
+
 	/* well normal icmp type should have at least 8 bytes, so this is min for us */
 	if (remain < 8) {
 		error("Can't load packet: Packet length shorter than ICMP header length!");
@@ -1001,11 +1209,14 @@ int icmp_header(GtkButton *button) {
 }
 
 
-int usedef_insert(GtkButton *button, char *entry) {
+int usedef_insert(GtkButton *button, char *entry, int whocalled) {
 
 	int i, j;
 	char tmp[4600];
 
+	if (whocalled == 1)
+		return 1;
+
 	/* get access to buffer of the text field */
 	w2 = lookup_widget(GTK_WIDGET(button), entry);
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w2));
@@ -1033,11 +1244,17 @@ int usedef_insert(GtkButton *button, char *entry) {
 }
 
 
-int tcp_header(GtkButton *button) {
+int tcp_header(GtkButton *button, int whocalled) {
 
 	int x, i, j;
 	char tmp[4600], tmp2[3], ch;
 
+	if (whocalled==2) {
+		protokol = TCP;
+		return 1;
+	}
+
+
 	/* for standard header this is minimum length */
 	if (remain < 20) {
 		error("Can't load packet: Packet length shorter than TCP header length!");
@@ -1069,7 +1286,7 @@ int tcp_header(GtkButton *button) {
 	insint(button, "entry49", ptrf, 8);
 	
 	/* now we insert value for length */
-	sprintf(tmp2, "%d", x*4);
+	snprintf(tmp2, 3, "%d", x*4);
 	w1 = lookup_widget(GTK_WIDGET(button), "entry50");
 	gtk_entry_set_text(GTK_ENTRY(w1), tmp2);
 
@@ -1159,11 +1376,16 @@ int tcp_header(GtkButton *button) {
 	return 1;
 }
 
-int udp_header(GtkButton *button) {
+int udp_header(GtkButton *button, int whocalled) {
 
 	int i, j;
 	char tmp[4600];
 
+	if (whocalled==2) {
+		protokol = UDP;
+		return 1;
+	}
+
 	/* for standard header this is minimum length */
 	if (remain < 8) {
 		error("Can't load packet: Packet length shorter than UDP header length!");
@@ -1226,11 +1448,15 @@ int udp_header(GtkButton *button) {
 }
 
 
-int ipv4_header(GtkButton *button) {
+int ipv4_header(GtkButton *button, int whocalled, struct clist_hdr *clptr ) {
 
-	char tmp[4];
+	char tmp[5];
 	int x, header_l, prot;
 
+	if (whocalled==2) {
+		protokol = IPv4;
+	}
+
 	/* for standard header this is minimum length */
 	if (remain < 20) {
 		error("Can't load packet: IPv4 header field is not long enough!");
@@ -1256,32 +1482,44 @@ int ipv4_header(GtkButton *button) {
 	}
 	ptrf--;
 
-	/* insert version */
-	inspar(button, "entry26", ptrf, 1);
+	if (whocalled==1) {
+		/* insert version */
+		inspar(button, "entry26", ptrf, 1);
 
-	/* insert header lenght */
-	inspar(button, "entry27", ptrf, 1);
+		/* insert header length */
+		inspar(button, "entry27", ptrf, 1);
 
-	/* insert tos */
-	inspar(button, "entry28", ptrf, 2);
+		/* insert tos */
+		inspar(button, "entry28", ptrf, 2);
+
+		/* insert total length */
+		w1 = lookup_widget(GTK_WIDGET(button), "checkbutton21");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+		//insint(button, "entry29", ptrf, 4);
+	}
+	else
+		ptrf = ptrf+4;
 
-	/* insert total length */
-	w1 = lookup_widget(GTK_WIDGET(button), "checkbutton21");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
-	//insint(button, "entry29", ptrf, 4);
 	ptrf = ptrf + 4;
 
-	/* insert identification */
-	inspar(button, "entry30", ptrf, 4);
+	if (whocalled==1) {
+		/* insert identification */
+		inspar(button, "entry30", ptrf, 4);
+	}
+	else
+		ptrf = ptrf+4;
 
 	/* insert flags */
 	*tmp = 0x30; /* 0x30 == 0 */
 	*(tmp+1) = *ptrf;
 	x = char2x(tmp);
 	x = x >> 1; /* use only first 3 bits */
-	w1 = lookup_widget(GTK_WIDGET(button), "entry31");
-        sprintf(tmp, "%d", x);
-        gtk_entry_set_text(GTK_ENTRY(w1), tmp);
+	
+	if (whocalled==1) {
+		w1 = lookup_widget(GTK_WIDGET(button), "entry31");
+        	snprintf(tmp, 4, "%d", x);
+        	gtk_entry_set_text(GTK_ENTRY(w1), tmp);
+	}
 
 	/* insert fragment offset */
 	*tmp = 0x30; /* 0x30 == 0 */
@@ -1292,62 +1530,83 @@ int ipv4_header(GtkButton *button) {
 	else
 		*tmp = 0x31;
 	strncpy(tmp+1, ptrf+1, 3);
-	insint(button, "entry32", tmp, 4);
+
+	if (whocalled==1) {
+		insint(button, "entry32", tmp, 4);
 	
-	/* insert ttl */
-	insint(button, "entry44", ptrf, 2);
+		/* insert ttl */
+		insint(button, "entry44", ptrf, 2);
+	}
+	else
+		ptrf = ptrf+6;
 
-	/* insert protocol */
 	prot = char2x(ptrf);
-	insint(button, "entry34", ptrf, 2);
 
-	/* insert header checksum */
-	w1 = lookup_widget(GTK_WIDGET(button), "ip_header_cks_cbt");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
-	//inspar(button, "entry35", ptrf, 4);
+	if (whocalled==1) {
+		/* insert protocol */
+		insint(button, "entry34", ptrf, 2);
+
+		/* insert header checksum */
+		w1 = lookup_widget(GTK_WIDGET(button), "ip_header_cks_cbt");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+		//inspar(button, "entry35", ptrf, 4);
+	}
+	else
+		ptrf = ptrf+2;
+
 	ptrf = ptrf + 4;
 
 	/*insert source ip */
 	ptrt = temp;
 	memset(temp, 0, 20);
-	w1 = lookup_widget(GTK_WIDGET(button), "entry38");
+	if (whocalled==1)
+		w1 = lookup_widget(GTK_WIDGET(button), "entry38");
 	for (i=1; i<=12; i++, ptrt++) {
 		if (i%3 == 0) {			
 			x = char2x(tmp);
 			if (i==12)
-				sprintf(tmp, "%d", x);		
+				snprintf(tmp, 4, "%d", x);		
 			else
-				sprintf(tmp, "%d.", x);		
+				snprintf(tmp, 5, "%d.", x);		
 			strcat(temp, tmp);
 		}
 		else {
 			tmp[(i-1)%3] = *ptrf++;
 		}
 	}
-	gtk_entry_set_text(GTK_ENTRY(w1), temp);
+
+	if (whocalled==1)
+		gtk_entry_set_text(GTK_ENTRY(w1), temp);
+	else
+		memcpy(clptr->src, temp, 20);
 
 	/*insert destination ip */
 	ptrt = temp;
 	memset(temp, 0, 20);
-	w1 = lookup_widget(GTK_WIDGET(button), "entry37");
+	if (whocalled==1)
+		w1 = lookup_widget(GTK_WIDGET(button), "entry37");
 	for (i=1; i<=12; i++, ptrt++) {
 		if (i%3 == 0) {			
 			x = char2x(tmp);
 			if (i==12)
-				sprintf(tmp, "%d", x);		
+				snprintf(tmp, 4, "%d", x);		
 			else
-				sprintf(tmp, "%d.", x);		
+				snprintf(tmp, 5, "%d.", x);		
 			strcat(temp, tmp);
 		}
 		else {
 			tmp[(i-1)%3] = *ptrf++;
 		}
 	}
-	gtk_entry_set_text(GTK_ENTRY(w1), temp);
+	if (whocalled==1)
+		gtk_entry_set_text(GTK_ENTRY(w1), temp);
+	else
+		memcpy(clptr->dst, temp, 20);
 
 	/* insert ipv4 options 
 	 * header_l * 4 == total header length, - 20 for standard header == options length in bytes*/
-	inspar(button, "entry39", ptrf, ( (header_l*4) - 20) * 2);
+	if (whocalled==1)
+		inspar(button, "entry39", ptrf, ( (header_l*4) - 20) * 2);
 
 	remain = remain - (header_l * 4);
 	
@@ -1355,11 +1614,16 @@ int ipv4_header(GtkButton *button) {
 }
 
 
-int ethernet_8023(GtkButton *button) {
+int ethernet_8023(GtkButton *button, int whocalled) {
 
 	int dsap, lsap, ctrl;
 	long pid;
 
+	if (whocalled==2) {
+		protokol = ETH_802_3;
+		return 1;
+	}
+
 	if (remain < 6) {
 		error("Can't load packet: Ethernet 802.3 LLC field is not long enough!");
 		return -1;
@@ -1437,12 +1701,17 @@ int ethernet_8023(GtkButton *button) {
 }
 
 
-int ethernet_verII(GtkButton *button) {
+int ethernet_verII(GtkButton *button, int whocalled) {
 
 	int pid;
 	
-	w1 = lookup_widget(GTK_WIDGET(button), "bt_ver2");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+	if (whocalled==2) 
+		protokol = ETH_II;
+
+	if (whocalled==1) {
+		w1 = lookup_widget(GTK_WIDGET(button), "bt_ver2");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+	}
 
 	pid = char2x(ptrf)*256 + char2x(ptrf+2);
 
@@ -1473,13 +1742,13 @@ void inspar(GtkButton *button, char *entry, char *from, int length) {
 } 
 
 
-/* this one reads (lengh) characters strating at (*from), converts them to int and inserts them 
+/* this one reads (length) characters strating at (char *from), converts them to int and inserts them 
 into field (*entry) as integer. f.e: 0x56 == (int)86 => writes into (*entry) 86 
 note that max size for length is 10!!! when calling this routine */
 void insint(GtkButton *button, char *entry, char *from, int length) {
 
 	GtkWidget *widg;
-	char tmp[10];
+	char tmp[11];
 	unsigned long value = 0;
 	int i;
 	unsigned char x = 0;
@@ -1500,7 +1769,7 @@ void insint(GtkButton *button, char *entry, char *from, int length) {
 
 	ptrf = ptrf + length;
 
-	sprintf(tmp, "%lu", value);
+	snprintf(tmp, 11, "%lu", value);
 	gtk_entry_set_text(GTK_ENTRY(widg), tmp);
 } 
 
@@ -1524,7 +1793,7 @@ signed int retint(char *ch) {
 }
 
 
-/* this one reads (lengh) characters strating at (*from), and returns integer (max 10 char length) */
+/* this one reads (length) characters strating at (*from), and returns integer (max 10 char length) */
 unsigned long retint2(char *from, int length) {
 
 	unsigned long value = 0;
@@ -1545,3 +1814,25 @@ unsigned long retint2(char *from, int length) {
 
 	return value;
 } 
+
+
+/* i have newer really understood the endians... help appreciated... 
+ * this routines just converts the contents of a char field of size 8 chars 
+ * it works also, if destination and source are the same field*/
+
+void convert8field(char *to, char *from) {
+
+	char f1[8];
+
+	/* we copy first the source contents */
+	memcpy(f1, from, 8);
+
+	memcpy(to+0, f1+6,1);
+	memcpy(to+1, f1+7,1);
+	memcpy(to+2, f1+4,1);
+	memcpy(to+3, f1+5,1);
+	memcpy(to+4, f1+2,1);
+	memcpy(to+5, f1+3,1);
+	memcpy(to+6, f1+0,1);
+	memcpy(to+7, f1+1,1);
+}
